/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams;

import java.util.List;
import java.util.Map;

/**
 * Exposes callback-record details for the current testrun.
 * A test will reach a proper instance of this interface by referencing it as
 * any other callback-interface, e.g. by having a field of this type
 * annotated with
 * {@link org.callbackparams.annotation.CallbackField @CallbackField}:
 * <pre>
 * &#64;CallbackField CallbackControlPanel controlPanel;
 * </pre>
 * @author Henrik Kaipe
 */
public interface CallbackControlPanel {

    /**
     * Returns the current callback-record as a list. - Changes to the list
     * will be propagated to the callback-record. affecting the subsequent
     * callback-invocations accordingly.
     */
    public List getCurrentCallbackRecord();

    /**
     * Returns the result (callback-record elements & return values) of the most
     * recent callback invocation as a map, where callback-record elements are
     * keys and return-values are values.<br/><br/>
     * For void-methods all the values will be null. Each primitive value will
     * be represented by an instance of its wrapper class.
     */
    public Map getLatestCallbackResults();
}
