/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.combine.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation allows the test-developer to combine callback-records
 * manually by annotating a static method and have it
 * return the desired collection of
 * explicit callback-records to use for the test.
 * <br/><br/>
 * The best practise for altering the combining is to configure a specific
 * combine-strategy by using the {@link CombineConfig @CombineConfig}
 * annotation. However, the combine-strategy approach does not make it easy to
 * predict the exact callback-records that will result from the combining, so
 * if there is a certain combination of callback-values that is of special
 * interest (because it often fails, represents an exceptional case etc) then
 * this annotation could prove useful.<br/>
 * When a test-class has a &#64;CallbackRecords-method it can still
 * configure a combine-strategy with a
 * {@link CombineConfig @CombineConfig} annotation.
 * The callback-records from the
 * &#64;CallbackRecords-method would then be tested first and
 * the combined records thereafter.
 *
 * @author Henrik Kaipe
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
@Documented
public @interface CallbackRecords {}
