/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.callbackparams.AdaptiveRule;
import org.callbackparams.CallbackControlPanel;
import org.callbackparams.annotation.CallbackField;
import org.callbackparams.internal.CallbackRebyteInfo;
import org.callbackparams.junit4.support.AdaptiveRuleAsJunitRule;

/**
 * @author Henrik Kaipe
 */
abstract class RebyteInfo extends DependentOnRulesSupport
implements CallbackRebyteInfo {

    private final Class<? extends Annotation>[] callbackMethodAnnotations;

    public RebyteInfo(Class<? extends Annotation>... callbackMethodAnnotations)
    {
        this.callbackMethodAnnotations = callbackMethodAnnotations;
    }

    public boolean isCallbackProxiedMethod(Method m) {
        final Class[] params = m.getParameterTypes();
        if (0 == params.length) {
            return false;
        }

        for (Class<?> p : params) {
            if (false == p.isInterface()) {
                return false;
            }
        }

        for (Class<? extends Annotation> a : callbackMethodAnnotations) {
            if (m.isAnnotationPresent(a)) {
                return true;
            }
        }

        return false;
    }

    /**
     * This implementation regards the first of the callback-method
     * annotations to be the test-method annotation.
     */
    public boolean isTestMethod(Method m) {
        return m.isAnnotationPresent(callbackMethodAnnotations[0]);
    }

    public boolean isCallbackInjectField(Field f) {
        if (false == f.isAnnotationPresent(CallbackField.class)) {
            return CallbackControlPanel.class == f.getType();
        } else if (f.getType().isInterface()) {
            return true;
        } else {
            throw new Error("Callback-field must be an interface (" + f + ')');
        }
    }

    public boolean isAdaptiveRuleField(Field f) {
        if (JUNIT_RULES_ARE_SUPPORTED && f.isAnnotationPresent(RULE_ANNOTATION))
        {
            return false;
        } else {
            return AdaptiveRule.class.isAssignableFrom(f.getType());
        }
    }

    public boolean isExplicitTopClass(Class clazz) {
        return false;
    }

    public Class adaptiveRuleSuperClass() {
        return JUNIT_RULES_ARE_SUPPORTED ? AdaptiveRuleAsJunitRule.class : null;
    }
}
