/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.support;

/**
 * @author Henrik Kaipe
 */
public class ThreadContext {

    public interface Testrun {
        void executeTestrun() throws Throwable;
    }

    public static void runWithCtxClassLoader(
            ClassLoader testrunClassLoader, Testrun testrun)
    throws Throwable {

        final ClassLoader originalThreadCtxClassLoader_Backup;

        try {
            originalThreadCtxClassLoader_Backup =
                    Thread.currentThread().getContextClassLoader();
            Thread.currentThread().setContextClassLoader(testrunClassLoader);

        } catch (SecurityException unableToSwitchThreadContextClassLoader) {
            testrun.executeTestrun();
            return; // Since the thread-context class-loader was never switched!
        }

        try {
            testrun.executeTestrun();

        } finally {
            /* Switch back */
            Thread.currentThread().setContextClassLoader(
                    originalThreadCtxClassLoader_Backup);
        }
    }
}
